//+----------------------------------------------------------------------------+
//|                                                             b-ForTest.mqh  |
//|                                                      . aka KimIV  |
//|                                                       http://www.kimiv.ru  |
//|                                                                            |
//|  13.06.2007        .   |
//+----------------------------------------------------------------------------+
#property library

// ClosePosBySelect    -  ,   
// CountOrders         -     
// DeleteOrders        -  
// ModifyOrder         -  
// OpenPosition        -  
// SetOrder            -  

//-------      -------------------+
color clOpenBuy, clOpenSell, clModifyBuy, clModifySell;
color clCloseBuy, clCloseSell, clDelete;
int   Slippage, StopLoss;

//-------    ------------------------------------------+
#include "b-Lots.mqh"        //   
#include "b-KimIV.mqh"       //   
#include <stdlib.mqh>

//+----------------------------------------------------------------------------+
//|     :   . aka KimIV,  http://www.kimiv.ru                    |
//+----------------------------------------------------------------------------+
//|    : 13.06.2007                                                      |
//|  :                      |
//+----------------------------------------------------------------------------+
void ClosePosBySelect() {
  double pp;
  int    dg=MarketInfo(OrderSymbol(), MODE_DIGITS);

  if (OrderType()==OP_BUY) {
    pp=NormalizeDouble(MarketInfo(OrderSymbol(), MODE_BID), dg);
    OrderClose(OrderTicket(), OrderLots(), pp, Slippage, clCloseBuy);
  }
  if (OrderType()==OP_SELL) {
    pp=NormalizeDouble(MarketInfo(OrderSymbol(), MODE_ASK), dg);
    OrderClose(OrderTicket(), OrderLots(), pp, Slippage, clCloseSell);
  }
}

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 03.08.2008                                                     |
//|   :     .                      |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    mo -                                      |
//|    mn - MagicNumber                          (-1 -  )            |
//+----------------------------------------------------------------------------+
void CountOrders(int& mo[], int mn=-1) {
  int i, k=OrdersTotal();

  if (ArraySize(mo)!=6) ArrayResize(mo, 6);
  for (i=0; i<k; i++) {
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {
      if (mn<0 || OrderMagicNumber()==mn) mo[OrderType()]++;
    }
  }
}

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|    : 13.06.2007                                                      |
//|   :  .      .        |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    sy -     (""  NULL -  )          |
//|    op -                    (    -1      -  )           |
//|    mn - MagicNumber                (    -1      -  )             |
//+----------------------------------------------------------------------------+
void DeleteOrders(string sy="", int op=-1, int mn=-1) {
  int i, k=OrdersTotal(), ot;

  if (sy=="" || sy=="0") sy=Symbol();
  for (i=0; i<k; i++) {
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {
      ot=OrderType();
      if (ot==OP_BUYLIMIT || ot==OP_BUYSTOP || ot==OP_SELLLIMIT || ot==OP_SELLSTOP) {
        if (OrderSymbol()==sy && (op<0 || ot==op)) {
          if (mn<0 || OrderMagicNumber()==mn) {
            OrderDelete(OrderTicket(), clDelete);
          }
        }
      }
    }
  }
}

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 28.03.2008                                                     |
//|   :  .      .      |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    pp -   ,                              |
//|    sl -                                                 |
//|    tp -                                                 |
//|    ex -                                                       |
//+----------------------------------------------------------------------------+
void ModifyOrder(double pp=-1, double sl=0, double tp=0, datetime ex=0) {
  int    dg=MarketInfo(OrderSymbol(), MODE_DIGITS), er;
  double op=NormalizeDouble(OrderOpenPrice() , dg);
  double os=NormalizeDouble(OrderStopLoss()  , dg);
  double ot=NormalizeDouble(OrderTakeProfit(), dg);
  color  cl;

  if (pp<=0) pp=OrderOpenPrice();
  if (sl<0 ) sl=OrderStopLoss();
  if (tp<0 ) tp=OrderTakeProfit();
  
  pp=NormalizeDouble(pp, dg);
  sl=NormalizeDouble(sl, dg);
  tp=NormalizeDouble(tp, dg);

  if (pp!=op || sl!=os || tp!=ot) {
    if (MathMod(OrderType(), 2)==0) cl=clModifyBuy; else cl=clModifySell;
    if (!OrderModify(OrderTicket(), pp, sl, tp, ex, cl)) {
      er=GetLastError();
      Print("Error(",er,") modifying order: ",ErrorDescription(er));
      Print("Ask=",Ask," Bid=",Bid," sy=",OrderSymbol(),
            " op="+GetNameOP(OrderType())," pp=",pp," sl=",sl," tp=",tp);
    }
  }
}

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 13.06.2007                                                     |
//|   :  .      .        |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    sy -     (""  NULL -  )          |
//|    op -                                                            |
//|    ll -                                                                 |
//|    sl -                                                         |
//|    tp -                                                         |
//|    mn - MagicNumber                                                        |
//+----------------------------------------------------------------------------+
void OpenPosition(string sy, int op, double ll, double sl=0, double tp=0, int mn=0) {
  color  clOpen;
  double pp;
  int    err, ticket;

  if (sy=="" || sy=="0") sy=Symbol();
  if (op==OP_BUY) {
    pp=MarketInfo(sy, MODE_ASK); clOpen=clOpenBuy;
  } else {
    pp=MarketInfo(sy, MODE_BID); clOpen=clOpenSell;
  }
  ticket=OrderSend(sy, op, ll, pp, Slippage, sl, tp, "", mn, 0, clOpen);
  if (ticket<0) {
    err=GetLastError();
    Print("Error(",err,") open ",GetNameOP(op),": ",ErrorDescription(err));
    Print("Ask=",Ask," Bid=",Bid," sy=",sy," ll=",ll,
          " pp=",pp," sl=",sl," tp=",tp," mn=",mn);
  }
}

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 13.06.2007                                                     |
//|   :  .      .        |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    sy -     (""  NULL -  )          |
//|    op -                                                            |
//|    ll -                                                                 |
//|    pp -                                                                |
//|    sl -                                                         |
//|    tp -                                                         |
//|    mn - Magic Number                                                       |
//|    ex -                                                       |
//+----------------------------------------------------------------------------+
void SetOrder(string sy, int op, double ll, double pp,
              double sl=0, double tp=0, int mn=0, datetime ex=0) {
  color cl=IIFc(op==OP_BUYLIMIT || op==OP_BUYSTOP, clOpenBuy, clOpenSell);
  int   err, ticket;

  if (sy=="" || sy=="0") sy=Symbol();
  ticket=OrderSend(sy, op, ll, pp, Slippage, sl, tp, "", mn, ex, cl);
  if (ticket<0) {
    err=GetLastError();
    Print("Error(",err,") set ",GetNameOP(op),": ",ErrorDescription(err));
    Print("Ask=",Ask," Bid=",Bid," sy=",sy," ll=",ll,
          " pp=",pp," sl=",sl," tp=",tp," mn=",mn);
  }
}
//+----------------------------------------------------------------------------+

