//+----------------------------------------------------------------------------+
//|                                                            b-Graphics.mqh  |
//|                                                                            |
//|                                                      . aka KimIV  |
//|                                                       http://www.kimiv.ru  |
//|                                                                            |
//|  12.10.2007        .       |
//+----------------------------------------------------------------------------+
#property library

// CrossPointOfLines       
// EquationDirect      
// GetArrowInterval             
// SetArrow              ,  OBJ_ARROW
// SetHLine             OBJ_HLINE  
// SetLabel             
// SetRegression        OBJ_REGRESSION   
// SetTLine             OBJ_TREND  
// SetTLineByAngle      OBJ_TRENDBYANGLE    
// SetVLine             OBJ_VLINE  

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 12.10.2007                                                     |
//|   :      .            |
//|                   .            |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    x -                x[0], x[1] -                |
//|                                    x[2], x[3] -                |
//|    y -                y[0], y[1] -                |
//|                                    y[0], y[1] -                |
//|    t -       t[0]       -                    |
//|                                    t[1]       -                    |
//+----------------------------------------------------------------------------+
void CrossPointOfLines(double& x[], double& y[], double& t[]) {
  double z=(y[3]-y[2])*(x[1]-x[0])-(y[1]-y[0])*(x[3]-x[2]);
  ArrayResize(t, 2);
  ArrayInitialize(t, 0.0);

  if (z==0) Print("CrossPointOfLines():     !");
  else {
    double xy1=x[1]*y[0]-x[0]*y[1];
    double xy2=x[3]*y[2]-x[2]*y[3];
    t[0]=NormalizeDouble((xy1*(x[3]-x[2])-xy2*(x[1]-x[0]))/z, 0);
    t[1]=(xy1*(y[3]-y[2])-xy2*(y[1]-y[0]))/z;
  }
}

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 12.10.2007                                                     |
//|   :  .                                              |
//|               Y  X     .       |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    x1,y1 -   ,                                        |
//|    x2,y2 -   ,                                        |
//|    x     - ,    Y                              |
//+----------------------------------------------------------------------------+
double EquationDirect(double x1, double y1, double x2, double y2, double x) {
  if (x2==x1) return(y1);
  return((y2-y1)/(x2-x1)*(x-x1)+y1);
}

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 23.10.2007                                                     |
//|   :                 |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    pr -                              |
//+----------------------------------------------------------------------------+
int GetArrowInterval(int pr=5) {
  if (pr<=0) pr=5;
  return((WindowPriceMax()-WindowPriceMin())/100*pr/Point);
}

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 12.10.2007                                                     |
//|   :    ,  OBJ_ARROW.                 |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    cd -                                                           |
//|    cl -                                                          |
//|    nm -                ("" -    )     |
//|    t1 -           (0  -  )                      |
//|    p1 -              (0  - Bid)                              |
//|    sz -                (0  -  )                     |
//+----------------------------------------------------------------------------+
void SetArrow(int cd, color cl,
              string nm="", datetime t1=0, double p1=0, int sz=0) {
  if (nm=="") nm=DoubleToStr(Time[0], 0);
  if (t1<=0) t1=Time[0];
  if (p1<=0) p1=Bid;
  if (ObjectFind(nm)<0) ObjectCreate(nm, OBJ_ARROW, 0, 0,0);
  ObjectSet(nm, OBJPROP_TIME1    , t1);
  ObjectSet(nm, OBJPROP_PRICE1   , p1);
  ObjectSet(nm, OBJPROP_ARROWCODE, cd);
  ObjectSet(nm, OBJPROP_COLOR    , cl);
  ObjectSet(nm, OBJPROP_WIDTH    , sz);
}

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 30.03.2008                                                     |
//|   :   OBJ_HLINE                 |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    cl -                                                           |
//|    nm -                ("" -    )     |
//|    p1 -              (0  - Bid)                              |
//|    st -                  (0  -  )                    |
//|    wd -                 (1  -  )                     |
//+----------------------------------------------------------------------------+
void SetHLine(color cl, string nm="", double p1=0, int st=0, int wd=1) {
  if (nm=="") nm=DoubleToStr(Time[0], 0);
  if (p1<=0) p1=Bid;
  if (ObjectFind(nm)<0) ObjectCreate(nm, OBJ_HLINE, 0, 0,0);
  ObjectSet(nm, OBJPROP_PRICE1, p1);
  ObjectSet(nm, OBJPROP_COLOR , cl);
  ObjectSet(nm, OBJPROP_STYLE , st);
  ObjectSet(nm, OBJPROP_WIDTH , wd);
}

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 12.10.2007                                                     |
//|   :   ,  OBJ_LABEL.                   |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    nm -                                                 |
//|    tx -                                                               |
//|    cl -                                                           |
//|    xd -  X                                              |
//|    yd -  Y                                              |
//|    cr -           (0 -   )                    |
//|                                     1 -                       |
//|                                     2 -                         |
//|                                     3 -   )                    |
//|    fs -                (9 -    )                    |
//+----------------------------------------------------------------------------+
void SetLabel(string nm, string tx, color cl, int xd, int yd, int cr=0, int fs=9) {
  if (ObjectFind(nm)<0) ObjectCreate(nm, OBJ_LABEL, 0, 0,0);
  ObjectSetText(nm, tx, fs);
  ObjectSet(nm, OBJPROP_COLOR    , cl);
  ObjectSet(nm, OBJPROP_XDISTANCE, xd);
  ObjectSet(nm, OBJPROP_YDISTANCE, yd);
  ObjectSet(nm, OBJPROP_CORNER   , cr);
  ObjectSet(nm, OBJPROP_FONTSIZE , fs);
}

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 12.10.2007                                                     |
//|   :   OBJ_REGRESSION   .     |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    cl -                                                           |
//|    nm -                ( ""   -    )  |
//|    t1 -           (  0   - Time[10])                      |
//|    t2 -           (  0   - Time[0])                       |
//|    ry -                         (False -  )                  |
//|    st -                  (  0   -  )                 |
//|    wd -                 (  1   -  )                  |
//+----------------------------------------------------------------------------+
void SetRegression(color cl, string nm="", datetime t1=0, datetime t2=0,
                    bool ry=False, int st=0, int wd=1) {
  if (nm=="") nm=DoubleToStr(Time[0], 0);
  if (t1<=0) t1=Time[10];
  if (t2<=0) t2=Time[0];
  if (ObjectFind(nm)<0) ObjectCreate(nm, OBJ_REGRESSION, 0, 0,0, 0,0);
  ObjectSet(nm, OBJPROP_TIME1, t1);
  ObjectSet(nm, OBJPROP_TIME2, t2);
  ObjectSet(nm, OBJPROP_COLOR, cl);
  ObjectSet(nm, OBJPROP_RAY  , ry);
  ObjectSet(nm, OBJPROP_STYLE, st);
  ObjectSet(nm, OBJPROP_WIDTH, wd);
}

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 12.10.2007                                                     |
//|   :   OBJ_TREND                      |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    cl -                                                           |
//|    nm -                (  ""  -    )  |
//|    t1 -           (  0   - Time[10]                       |
//|    p1 -              (  0   - Low[10])                       |
//|    t2 -           (  0   -  )                   |
//|    p2 -              (  0   - Bid)                           |
//|    ry -                         (False -  )                        |
//|    st -                  (  0   -  )                 |
//|    wd -                 (  1   -  )                  |
//+----------------------------------------------------------------------------+
void SetTLine(color cl, string nm="",
              datetime t1=0, double p1=0, datetime t2=0, double p2=0,
              bool ry=False, int st=0, int wd=1) {
  if (nm=="") nm=DoubleToStr(Time[0], 0);
  if (t1<=0) t1=Time[10];
  if (p1<=0) p1=Low[10];
  if (t2<=0) t2=Time[0];
  if (p2<=0) p2=Bid;
  if (ObjectFind(nm)<0) ObjectCreate(nm, OBJ_TREND, 0, 0,0, 0,0);
  ObjectSet(nm, OBJPROP_TIME1 , t1);
  ObjectSet(nm, OBJPROP_PRICE1, p1);
  ObjectSet(nm, OBJPROP_TIME2 , t2);
  ObjectSet(nm, OBJPROP_PRICE2, p2);
  ObjectSet(nm, OBJPROP_COLOR , cl);
  ObjectSet(nm, OBJPROP_RAY   , ry);
  ObjectSet(nm, OBJPROP_STYLE , st);
  ObjectSet(nm, OBJPROP_WIDTH , wd);
}

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 12.10.2007                                                     |
//|   :   OBJ_TRENDBYANGLE         |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    cl -                                                           |
//|    nm -                (  ""  -    )  |
//|    t1 -           (  0   - Time[10]                       |
//|    p1 -              (  0   - Low[10])                       |
//|    t2 -           (  0   -  )                   |
//|    p2 -              (  0   -  )                       |
//|    an -                        (  0   -  )                  |
//|    ry -                         (False -  )                        |
//|    st -                  (  0   -  )                 |
//|    wd -                 (  1   -  )                  |
//+----------------------------------------------------------------------------+
void SetTLineByAngle(color cl, string nm="",
              datetime t1=0, double p1=0, datetime t2=0, double p2=0,
              double an=0, bool ry=False, int st=0, int wd=1) {
  if (nm=="") nm=DoubleToStr(Time[0], 0);
  if (t1<=0) t1=Time[0];
  if (p1<=0) p1=Bid;
  if (t2<=0) t2=Time[0];
  if (ObjectFind(nm)<0) ObjectCreate(nm, OBJ_TRENDBYANGLE, 0, 0,0);
  ObjectSet(nm, OBJPROP_TIME1 , t1);
  ObjectSet(nm, OBJPROP_PRICE1, p1);
  ObjectSet(nm, OBJPROP_TIME2 , t2);
  if (p2>0) ObjectSet(nm, OBJPROP_PRICE2, p2);
  else ObjectSet(nm, OBJPROP_ANGLE, an);
  ObjectSet(nm, OBJPROP_COLOR, cl);
  ObjectSet(nm, OBJPROP_RAY  , ry);
  ObjectSet(nm, OBJPROP_STYLE, st);
  ObjectSet(nm, OBJPROP_WIDTH, wd);
}

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 02.07.2008                                                     |
//|   :   OBJ_VLINE                   |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    cl -                                                           |
//|    nm -                ("" -    )     |
//|    t1 -                       (0  -    )     |
//|    st -                  (0  -  )                    |
//|    wd -                 (1  -  )                     |
//+----------------------------------------------------------------------------+
void SetVLine(color cl, string nm="", datetime t1=0, int st=0, int wd=1) {
  if (nm=="") nm=DoubleToStr(Time[0], 0);
  if (t1<=0) t1=Time[0];
  if (ObjectFind(nm)<0) ObjectCreate(nm, OBJ_VLINE, 0, 0,0);
  ObjectSet(nm, OBJPROP_TIME1, t1);
  ObjectSet(nm, OBJPROP_COLOR, cl);
  ObjectSet(nm, OBJPROP_STYLE, st);
  ObjectSet(nm, OBJPROP_WIDTH, wd);
}
//+----------------------------------------------------------------------------+

